package javarequirementstracer.mojo;

import java.io.File;
import java.io.FileNotFoundException;

import javarequirementstracer.FileUtils;
import javarequirementstracer.JavaRequirementsTracer;
import javarequirementstracer.ReportAggregator;
import javarequirementstracer.Requirements;

import org.apache.maven.plugin.AbstractMojo;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;
import org.apache.maven.project.MavenProject;

/**
 * @author Ruud de Jong
 * @goal aggregate
 * @phase test
 * @requiresDependencyResolution test
 */
public class AggregatorMojo extends AbstractMojo {
    /**
     * Comma separated list of properties filenames. For each an overview report will be generated.
     * Default: {@value ReportAggregator#DEFAULT_OVERVIEW_PARAMS_FILENAME}.
     * If a file does not exist the plugin will skip generating the report and print a log message.
     *
     * @parameter expression="${aggregateParamsFilenames}"
     */
    private String aggregateParamsFilenames;

    /**
     * The Maven Project Object
     *
     * @parameter expression="${project}"
     * @required
     * @readonly
     */
    private MavenProject project;

    /**
     * Maven plugin execute method.
     * This will check whether the aggregator should be run,
     * @throws MojoExecutionException when an error occurs.
     */
    @Requirements({"UC-Goal-Aggregate", "UC-No-Spring"})
    public void execute() throws MojoExecutionException, MojoFailureException {
		info("Project basedir = " + this.project.getBasedir());
        info("Aggregate params filename = " + this.aggregateParamsFilenames);
		
        if (this.aggregateParamsFilenames == null) {
			this.aggregateParamsFilenames = ReportAggregator.DEFAULT_OVERVIEW_PARAMS_FILENAMES;
		}
        
		String[] paramsFilenames = JavaRequirementsTracer.split(this.aggregateParamsFilenames);		
		for (String paramsFilename : paramsFilenames) {
			if (!FileUtils.isAbsolute(paramsFilename)) {
				paramsFilename = this.project.getBasedir() + "/" + paramsFilename;
			}
			File paramFile = new File(paramsFilename);
			if (!paramFile.exists()) {
				info("Skipping aggregation for " + paramFile.getAbsolutePath() + " (file not found).");
				continue;
			}

            ReportAggregator aggregator = new ReportAggregator();
            aggregator.setOverviewParamsFilename(paramsFilename);
            try {
                aggregator.run();
            } catch (FileNotFoundException ex) {
                throw new MojoExecutionException("Could not load params file: " + paramsFilename, ex);
            }
		}
    }
    
    private void info(String str) {
		String prefix = "JRT-Aggregate: ";
		getLog().info(prefix + str);
    }
}
